/* 
 * File:   main.cpp
 * Author: Mohsen Reisi
 *
 * Created on 18 April 2015, 10:03 AM
 */

/*
 * This programme is created: 
 * 1- to read the digits from the zipped files and to allocate them to a vector,
 * 2- to count the 9-tuples with at least 5 consecutive identical digits like 124666663,
 * 3- to shuffle the digits and again to count the specific 9-tuples.
 */

#include <iostream>
#include <vector>
#include <fstream>
#include <string>
#include <sstream>
#include <ctime> 
#include <algorithm>
#include<iomanip>

#include "randomc/sfmt.h"
#include "randomc/sfmt.cpp"

using namespace std;

//This function counts the number of specific 9-tuples in vector digits. 
int countTuples(vector<char> digits)
{
    clock_t t1 = clock();
    int nTuples = digits.size() / 9;
    int pos, count = 0;
    string tuple;
    string rep;
    for (int i = 0; i < nTuples; ++i)
    {
        //9-tuple
        for (char j = 0; j < 9; ++j)
            tuple += digits[i * 9 + j];

        for (char c = '0'; c <= '9'; ++c)
        {
            //rep is a string of consecutive identical digits like "33333"
            for (int r = 0; r < 5; ++r)
                rep += c;
            //pos is the position of rep in the tuple 
            pos = tuple.find(rep);
            //if the tuple includes any rep
            if ((pos >= 0)&&(pos <= 6))
            {
                ++count;
                rep.clear();
                break;
            }
            rep.clear();
        }
        tuple.clear();
    }
    cout << count << " 9-tuples" << "   "
            << float(clock() - t1) / CLOCKS_PER_SEC << " sec" << endl;
    return (count);
}
//This function shuffle the digits within block bIndex that are allocated to vector digits.
void shuffleDigits(vector<char>& digits, int bIndex)
{
    clock_t t1 = clock();
    CRandomSFMT1 mySFMT(bIndex * (unsigned) time(0));
    int n = digits.size(), randN1, randN2;
    for (int i = 0; i < n / 2; ++i)
    {
        randN1 = mySFMT.IRandomX(0, n - 1);
        randN2 = mySFMT.IRandomX(0, n - 1);
        swap(digits[randN1], digits[randN2]);
    }
    cout << "SFMT1 shuffling is done!" << "   "
            << float(clock() - t1) / CLOCKS_PER_SEC << " sec" << endl;
    return;
}

/*
This function extracts the digits of block bIndex from the zipped files and
counts the specific 9-tuples within the block. 
*/
void testDigits(int& count1, int& count2, int bIndex, int nFiles)
{
    int fFile = (bIndex - 1) * nFiles + 1;
    int lFile = bIndex * nFiles;

    int i = fFile;
    while (i <= lFile)
    {
        clock_t t1 = clock();
        vector<char> digits;
        for (int j = 0; j < 9; ++j)
        {

            string txtFileName = "pi-";
			string zipFileName = "set";
                        if(((i-1)/10000+1)<=9)
                            zipFileName+="0";
            ostringstream temp;
			temp << (i-1)/10000+1;
			zipFileName += temp.str();
			zipFileName += "/pi-";
			temp.str("");
			temp.clear();
            temp << setfill('0') << setw(4) << i;

            txtFileName += temp.str();
            zipFileName += temp.str();

            txtFileName += ".txt";
            zipFileName += ".zip";

            string commandScript = "unzip -o " + zipFileName;
            if (system(commandScript.c_str()))
            {
                cout << "the zip file was not extracted properly!" << endl;
                exit(0);
            }
            ifstream digitsFile(txtFileName.c_str());
            string aLine;
            int nLines = 0;
            while (getline(digitsFile, aLine, '\n'))
            {
                ++nLines;
                int j = 0;
                while (aLine[j] != ':')
                {
                    if (aLine[j] != ' ')
                        digits.push_back(aLine[j]);
                    ++j;
                }
                aLine.clear();
            }
            digitsFile.close();
            if (nLines != 1000000)
            {
                cout << "Error: number of lines in the text file is wrong!" << endl;
                exit(0);
            }
            commandScript = "rm " + txtFileName;
            if (system(commandScript.c_str()))
            {
                cout << "the txt file was not deleted!" << endl;
                exit(0);
            }
            ++i;
        }
        cout << "number of digits= " << digits.size() << "   "
                << float(clock() - t1) / CLOCKS_PER_SEC << " sec" << endl;
        count1 += countTuples(digits);
        //shuffleDigits(digits, bIndex);
        //count2 += countTuples(digits);
        digits.clear();
    }
    return;
}
/*
This function tests every block of digits within {fBlock,...,lBlock}. 
The digits in each block are extracted from nFiles zipped text files. 
The output of the test of block 'i' is printed in text file "i.txt".
*/
void allBlocks(int fBlock,int lBlock,int nFiles)
{
    clock_t t1 = clock();
    cout.precision(2);
    cout << fixed;
    for (int bIndex = fBlock; bIndex <= lBlock; ++bIndex)
    {
        int count1 = 0, count2 = 0;
        string fileName = "";
        ostringstream temp;
        temp << bIndex;
        fileName += temp.str();
        fileName += ".txt";
        ofstream outFile(fileName.c_str());
        testDigits(count1, count2, bIndex, nFiles);
        outFile << count1 << ' ';
        outFile << count2 << endl;
        cout << "count1= " << count1 << '\t'
                << "count2= " << count2 << '\t'
                << "total time= "
                << float(clock() - t1) / CLOCKS_PER_SEC << " sec" << endl;
        outFile.close();
    }
}

/*
The inputs are the index of the first block (fBlock), the index of the last block (lBlock),
and the number of text files within each block. 
Example: ./getCounts 1 120 450 is the command to test blocks 1 to 120 where there are 450 text files wthin each block (there are 4.50E+10 digits within each block). 
*/  
int main(int argc, char * argv[])
{
    int fBlock = atoi(argv[1]);
    int lBlock = atoi(argv[2]);
    int nFiles = atoi(argv[3]);
    allBlocks(fBlock, lBlock, nFiles);
    return 0;
}
